/***************************************************************************************************
*****    This file is part of KardsGT.                                                         *****
*****                                                                                          *****
*****    Copyright (C) 2008  John Schneiderman <JohnMS@member.fsf.org>                         *****
*****                                                                                          *****
*****    This program is free software: you can redistribute it and/or modify it under the     *****
*****    terms of the GNU General Public License as published by the Free Software Foundation, *****
*****    either version 3 of the License, or (at your option) any later version.               *****
*****                                                                                          *****
*****    This program is distributed in the hope that it will be useful, but WITHOUT ANY       *****
*****    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A       *****
*****    PARTICULAR PURPOSE.  See the GNU General Public License for more details.             *****
*****                                                                                          *****
*****    You should have received a copy of the GNU General Public License along with this     *****
*****    program. If not, see <http://www.gnu.org/licenses/>.                                  *****
***************************************************************************************************/
#ifndef DEMO_H
#define DEMO_H

#include "gamebase.h"
#include "demorules.h"

class DemoInterface;

/**
 * This controls and keeps track of the state of the game demo.
 *
 * @author YOUR_NAME_HERE
 */
class Demo: public GameBase
{
public:
    /**
     * The constructor for the demo.
     * @param gameInterface is the interface for the game.
     * @param profileDatabase is the user profile database.
     */
    Demo(DemoInterface *gameInterface, UserProfileDatabase &profileDatabase);
    /**
     * The destructor.
     */
    ~Demo();
    /**
     * This starts the game of demo.
     */
    virtual void startPlay();
    /**
     * This saves the game of demo.
     * @param filename is the name of the file to save the game to.
     * @return true if we successfully save the game, false elsewise.
     */
    virtual bool save(const QString &filename);
    /**
     * This loads the game of demo.
     * @param filename is the name of the file to load.
     * @return true if we load the game successfully, false elsewise.
     */
    virtual bool load(const QString &filename);
    /**
     * This gives the minimum number of players needed to play the game of demo.
     * @return the minimum number of players.
     */
    virtual int minimumPlayers() const;
    /**
     * This gives the maximum number of players allowed to play the game of demo.
     * @return the maximum number of players.
     */
    virtual int maximumPlayers() const;

protected:
    /// @param m_rules is the rules to the game of demo.
    DemoRules m_rules;

    /**
     * Handles a card being played.
     * @param card is the card the player wants to play.
     * @return true if the card was successfully played, false if it was not.
     */
    virtual bool cardPlayed(const Card &card);
    /**
     * Deals out the cards to each player.
     */
    virtual void deal();
    /**
     * Disables the current player, and enables the next player's turn.
     */
    virtual void setupNextPlayer();
    /**
     * Determines and handles the various stages in game play.
     * @return the last major GameStatusCode value after handling the changes.
     */
    virtual int handleGameStatus();

private:
    /// @param m_pInterface is the interface to our graphical interface.
    DemoInterface *m_pInterface;
};
#endif
