/***************************************************************************************************
*****    This file is part of KardsGT.                                                         *****
*****                                                                                          *****
*****    Copyright (C) 2008  John Schneiderman <JohnMS@member.fsf.org>                         *****
*****                                                                                          *****
*****    This program is free software: you can redistribute it and/or modify it under the     *****
*****    terms of the GNU General Public License as published by the Free Software Foundation, *****
*****    either version 3 of the License, or (at your option) any later version.               *****
*****                                                                                          *****
*****    This program is distributed in the hope that it will be useful, but WITHOUT ANY       *****
*****    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A       *****
*****    PARTICULAR PURPOSE.  See the GNU General Public License for more details.             *****
*****                                                                                          *****
*****    You should have received a copy of the GNU General Public License along with this     *****
*****    program. If not, see <http://www.gnu.org/licenses/>.                                  *****
***************************************************************************************************/
#ifndef DEMORULES_H
#define DEMORULES_H

#include "rulebase.h"

class CardSequence;

/**
 * The rules for a game of demo.
 *
 * @author YOUR_NAME_HERE
 */
class DemoRules : public RuleBase
{
public:
    /**
     * Default constructor.
     */
    DemoRules();
    /**
     * Default destructor.
     */
    ~DemoRules();
    /**
     * This determines if the conditions have been met for the game to end.
     * @param players are the players in the game.
     * @return true if the conditions have been met, false elsewise.
     */
    virtual bool isGameOver(const PlayerList& players) const;
    /**
     * This determines if a card being played is a legal play.
     * @param sequence is the current play sequence.
     * @param cardPlayed is the card the player wants to player.
     * @param player is the player who played the card.
     * @return true if the card played is a legal play, false elsewise.
     */
    virtual bool isLegalPlay(const CardSequence& sequence, const Card& cardPlayed, const Player& player) const;
    /**
     * This determines if the conditions have been met for the phase to end.
     * @param players are the players in the game.
     * @param playSequence is the current play sequence.
     * @return true if the conditions have been met, false elsewise.
     */
    virtual bool isPhaseOver(const PlayerList &players, const CardSequence &playSequence) const;
    /**
     * This determines if the conditions have been met for the round to end.
     * @param players are the players in the game.
     * @return true if the conditions have been met, false elsewise.
     */
    virtual bool isRoundOver(const PlayerList& players) const;
    /**
     * This gives the card point value.
     * @param card is the card we want to know the value of.
     * @return the point value of the card.
     */
    virtual int cardValue(const Card& card) const;
    /**
     * Gives the rank order value for a card.
     * @param card is the card we wish to examine.
     * @return the rank value of the card.
     */
    virtual int rankValue(const Card&) const;
    /**
     * Gives the maximum number of players allowed in the game.
     * @return the maximum number of players.
     */
    virtual int maximumNumberOfPlayers() const;
    /**
     * Gives the minimum number of players need for the game.
     * @return the minimum number of players.
     */
    virtual int minimumNumberOfPlayers() const;
    /**
     * Gives the number of cards to deal to each player.
     * @param numberOfPlayers is the number of players in the game.
     * @return the number of cards to deal.
     */
    virtual int numberOfCardsToDeal(int numberOfPlayers = -1) const;
    /**
     * Gives the score needed to win the game.
     * @param numberOfPlayers is the number of players in the game.
     * @return Gives the score needed to "win" the game.
     */
    virtual int winningGameScore(int numberOfPlayers = -1) const;
};
#endif
